%% Evaluate the accuracy of our push-approximation to the F-measure

addpath('../../matlab');
addpath('../../tools');
addpath('../');

%% load a graph
if ~exist('graphname','var') || ~exist('A','var')
    error('need to load a graph first another function');
end
%% normalize the graph
A = A|A';
A = A - diag(diag(A));
deg = sum(A,1)';
D = diag(sparse(deg));
L = D - A;
n = size(A,1);

%% Pick a vertex
vert = pairs_conv(2);

%% Result filename
rfilename = sprintf('accuracy-results-%s-%i',graphname,vert);

%% Solve the system with minres
Ax = @(x) L*x + 1/n*sum(x);
ei = zeros(n,1);
ei(vert) = 1;
[x,flag,relres,iter,resvec] = minres(Ax,ei,1e-8,5000);
if flag ~= 0,
    error('minres did not converge');
end
f = deg.*x + deg(vert)*x;
xc = x - mean(x);
fc = deg.*xc + deg(vert)*xc;
[ignore forder] = sort(-f);
%% Solve the system with our Lp-push operator
nsteps = [100,1000,2500,5000,10000,0.1*n, 0.5*n n,2*n,5*n,10*n];
ks = [10,25,100,1000];
nsteps = sort(nsteps);
results = [];
for ni=1:length(nsteps)
    nstep = nsteps(ni);
    %%
    [x,r,hist,nverts] = Lp_push_markov_mex(A,vert,1,1/n,nstep);
    xc = x - mean(x);
    fp = deg.*x + deg(vert)*x;
    fpc = deg.*xc + deg(vert)*xc;
    [ignore fporder] = sort(-fp);
    results(ni).diff = norm(fc-fpc);
    results(ni).cos = fc'*fpc/(norm(fc)*norm(fpc));
    results(ni).ktau = ktau(fc,fpc);
    ktauk = zeros(length(ks),1);
    overlapk = ktauk; % copy the zero values
    for ki=1:length(ks);
        k = ks(ki);
        ktauk(ki) = ktau(fp(forder(1:k)),f(forder(1:k)));
        overlapk(ki) = length(intersect(forder(1:k),fporder(1:k)))./k;
    end
    results(ni).ktauk = ktauk;
    results(ni).overlapk = overlapk;
    
    results(ni).fp = fp;
    results(ni).r = r;
    results(ni).hist = hist;
    results(ni).nverts = nverts;
    results(ni).nedges = hist(1,end);
    clf; hold on; 
	allktau = [results(1:ni).ktauk];
    alloverlap = [results(1:ni).overlapk];
    for ki=1:length(ks);
        semilogx([results(1:ni).nedges]/nnz(A),allktau(ki,:),'.-');
        semilogx([results(1:ni).nedges]/nnz(A),alloverlap(ki,:),'r.-');
    end
    set(gca,'XScale','log');
    drawnow;
end

save(rfilename,'results','ks','nsteps','vert');

%% Use CG for baseline
nmults=[2 5 10 15 25 50];
cgresults =[];
for ni=1:length(nmults)
    nmult=nmults(ni);
    [x,flag,relres,iter,resvec] = pcg(Ax,ei,0,nmult);
    fcg = deg.*x + deg(vert)*x;
    xc = x - mean(x);
    fcgc = deg.*xc + deg(vert)*xc;
    [ignore fcgorder] = sort(-fcg);
    
    cgresult.diff = norm(fc-fcgc);
    cgresult.cos = fc'*fcgc/(norm(fc)*norm(fcgc));
    cgresult.ktau = ktau(fc,fcgc);
    
    k = 25;
    ktauk = ktau(fcg(forder(1:k)),f(forder(1:k)));
    overlapk = length(intersect(forder(1:k),fcgorder(1:k)))./k;
    
    cgresult.ktauk = ktauk;
    cgresult.overlapk = overlapk;
    
    cgresult.fcg = fcg;
    cgresult.nverts = n;
    cgresult.nedges = nmult*nnz(A);
    
    if ni==1
        cgresults=cgresult;
    else
        cgresults(ni)=cgresult;
    end
end

%%
save(sprintf('accuracy-results-%s-%i',graphname,vert),...
    'results','ks','nsteps','vert','cgresults','nmults');

%% Load the data we just saved
load(rfilename);

%% Plot the convergence in the ordering
setup_plot

clf; hold on;
allktau = [results.ktauk];
alloverlap = [results.overlapk];
xs = [results.nedges]/nnz(A);
legtexts = {};
hs=[];
styles = {'.-','.--','.-.','.:'};
for ki=1:length(ks);
    hs(ki) = semilogx(xs,allktau(ki,:),styles{ki},'MarkerSize',14,...
        'Color',0.5*[1,1,1],'linewidth',1.1);
    legtexts{ki} = sprintf('k=%i ',ks(ki));
end
% double-print k=25
hs(end+1) = semilogx([cgresults.nedges]/nnz(A),[cgresults.ktauk],'ko-');
legtexts{end+1} = 'cg k=25 ';
ki=2;
hs(end+1) = semilogx(xs,allktau(ki,:),styles{ki},'MarkerSize',14,'Color','r');
legtexts{end+1} = sprintf('k=%i ',ks(ki));
legtexts(ki)=[];
hs(ki)=[];

ylim([-0.1,1.1]);
xlim([1e-3,1e3]);
lh = legend(hs,legtexts{:},'Location','SE');
legend boxoff;
set(lh,'FontSize',18);
set(gca,'XScale','log');
xlabel('Equivalent matrix-vector products');
ylabel('Kendtall-\tau ordering vs. exact');

%%
box on;
greygrid;
set(gca,'Color','none'), set(gcf,'Color','none');
set(gcf, 'InvertHardCopy','off');
print(gcf,sprintf('%s-topk-ktau-%i.eps',graphname,vert),'-depsc2');

%% Plot the convergence in the sets
clf; hold on;
alloverlap = [results(1:length(results)).overlapk];
xs = [results(1:length(results)).nedges]/nnz(A);
legtexts = {};
hs=[];
styles = {'.-','.--','.-.','.:'};
for ki=1:length(ks);
    hs(ki) = semilogx(xs,alloverlap(ki,:),styles{ki},'MarkerSize',14,...
        'Color',0.5*[1,1,1],'linewidth',1.1);
    % againth, the space is important'
    %text(xs(1),alloverlap(ki,1),sprintf('k=%i ',ks(ki)),'HorizontalAlignment','right');
    legtexts{ki} = sprintf('k=%i ',ks(ki));
end
% double-print k=25
hs(end+1) = semilogx([cgresults.nedges]/nnz(A),[cgresults.overlapk],'ko-');
legtexts{end+1} = 'cg k=25 ';
ki=2;
hs(end+1) = semilogx(xs,alloverlap(ki,:),styles{ki},'MarkerSize',14,'Color','r');
legtexts{end+1} = sprintf('k=%i ',ks(ki));
legtexts(ki)=[];
hs(ki)=[];
set(gca,'XScale','log');
ylim([-0.1,1.1]);
xlim([1e-3,1e3]);
lh = legend(hs,legtexts{:},'Location','SE');
legend boxoff;
set(lh,'FontSize',18);

xlabel('Equivalent matrix-vector products');
ylabel('Precision@k for exact top-k sets');

%%
box on;
greygrid;
set(gca,'Color','none'), set(gcf,'Color','none');
set(gcf, 'InvertHardCopy','off');
print(gcf,sprintf('%s-topk-overlap-%i.eps',graphname,vert),'-depsc2');