%% Bidirectional diffusion affinity motivation
% To motivate the bidirectional diffusion affinity scores, we 
% show the relationship between them and commute time for a small line
% graph.

nverts = 25;
e = ones(nverts,1);
A = spdiags([e,e],[-1,1],nverts,nverts);
n = size(A,1);
deg = sum(A,1)';

%% Compute the pseudo-inverse
D = diag(sum(A,1)');
L = D-A;
Lp = pinv(full(L)); % just compute the pseudo-inverse
P = diag(1./deg)*A;


%% Compute commute time values
% build commute time column-by-column... todo: find a matrix way
C = zeros(n);
for i=1:n
    C(:,i) = diag(Lp) + Lp(i,i) - 2*Lp(:,i);
end

%% Compute our F-measure
% negative to give relationship with commute time
F = D*Lp;
F = -(F + F');
%% Plot
set(0, ...
    'defaultaxesfontsize',   25, ...
    'defaultaxeslinewidth',   1.5, ...
    'defaultlinelinewidth',   2, ...
    'defaultpatchlinewidth',  1.1);

clf; hold on; axis off;
% assign coords for the grpah
xy = zeros(n,2);
xy(:,1) = 1:nverts;
% get info to plot it
[gx,gy] = gplot(A,xy);
% plot it with nice big markers
markersize = 18;
plot(gx,gy,'k.-','MarkerSize',markersize);
vert = 12;
plot(xy(vert,1),xy(vert,2),'.','MarkerSize',markersize,'Color','red');
text(n+1,0,'Line graph','FontSize',20);
xlim([0,35])


yscale = 2;
yskip = 1;

% plot the commute time
ct = C(vert,:);
ct = ct-min(ct);
ct = ct/max(ct);
plot(xy(:,1),ct*yscale+yskip,'k.-','MarkerSize',markersize);
text(n+1,yskip+yscale/2,'Commute Time','FontSize',20);

% plot the f time
ct = F(vert,:);
ct = ct-min(ct);
ct = ct/max(ct);
plot(xy(:,1),ct*yscale+(2*yskip+yscale),'k.-','MarkerSize',markersize);
text(n+1,(2*yskip+yscale)+yscale/2,'F-measure','FontSize',20);


set(gca,'Color','none'), set(gcf,'Color','none');
set(gcf, 'InvertHardCopy','off');
print(gcf,sprintf('F_motivation.eps'),'-depsc2');

