%% Evaluate topk commute time via two different measures on a big/smallish graph

addpath('../../matlab');
addpath('../../gaimc');
%% Real world graph
% graphname = 'cs-stan';
% graphbase = '../../data/stanford-cs-sym.smat';

if ~exist('graphname','var')
    graphname = 'ca-HepTh';
    graphbase = '../../data/ca-HepTh-cc.smat';
end
%%

A = readSMAT(graphbase);
C = load_scalapack_matrix([graphbase '.laplacian.ctimes.gz']);
n = size(A,1);
deg = sum(A,1)';


%% Pick a set of random nodes and plot the precision of finding the closest non-neigbhor nodes
nnodes = 300;
rand('state',0);
randnodes = randperm(n);
nodes = randnodes(1:nnodes);
ks = [5:5:100];
targetk = 25;
precisions_ap = zeros(nnodes, length(ks));
precisions_F = zeros(nnodes, length(ks));
precisions_D = precisions_F;
recall_D = precisions_D;
recall_F = precisions_F;
recall_ap = precisions_ap;
for ni=1:nnodes
    curn = nodes(ni);
    c = C(:,curn);
    t0 = tic;
    [ap,d,reshist,x] = commute_topk(A,curn,'tol',1e-16); % approx commute
    dt_l = toc(t0);
    t0 = tic;
    
    %fv = 1./deg;
    %[fv,stats,xp] = F_topk(A,curn); % F score
    
    %x = x-min(x);
    %fv = -(deg.*x + (deg(curn)).*x);
    
    % test with 2./deg(i) as the diagonal element
    fv = 2./deg+x(curn)-2*x;
    
    dt_p = toc(t0);
    d = 1./deg;
    fprintf('Runtime for node %5i (%2i/%2i): lanczos=%5.2f sec   lanczos_f=%5.2f sec\n', ...
        curn, ni, nnodes, dt_l, dt_p);
    
    c(curn) = max(c);
    c(A(curn,:)>0) = max(c);
    
    ap(curn) = max(ap);
    ap(A(curn,:)>0) = max(ap);
    
    fv(curn) = max(fv);
    fv(A(curn,:)>0) = max(fv);
    
    d(curn) = max(d);
    d(A(curn,:)>0) = max(d);
    
    [ignore pC] = sort(c);
    [ignore pL] = sort(ap);
    [ignore pF] = sort(fv);
    [ignore pD] = sort(d);
    
    for ki=1:length(ks)
        k = ks(ki);
        %precisions(ni,ki) = length(intersect(pL(1:k),pC(1:k)))/k;
        recall_ap(ni,ki) = length(intersect(pL(1:k),pC(1:targetk)))/min(k,targetk);
        recall_F(ni,ki) = length(intersect(pF(1:k),pC(1:targetk)))/min(k,targetk);
        recall_D(ni,ki) = length(intersect(pD(1:k),pC(1:targetk)))/min(k,targetk);
        precisions_ap(ni,ki) = length(intersect(pL(1:k),pC(1:k)))/min(k,k);
        precisions_F(ni,ki) = length(intersect(pF(1:k),pC(1:k)))/min(k,k);
        precisions_D(ni,ki) = length(intersect(pD(1:k),pC(1:k)))/min(k,k);
    end
    
    save(sprintf('%s-precision-results.mat',graphname),...
        'precisions_F','precisions_ap','precisions_D',...
        'recall_ap','recall_F','recall_D','nodes','targetk','ks','nnodes');
end
commute_topk_plot