%% Evaluate topk commute time via two different measures on a big/smallish graph

addpath('../../matlab');
addpath('../../gaimc');
%% Real world graph
% graphname = 'cs-stan';
% graphbase = '../../data/stanford-cs-sym.smat';

if ~exist('graphname','var')
    graphname = 'enron';
    graphbase = '../../data/email-Enron-cc.smat';
end

%%

A = readSMAT(graphbase);
C = readSMAT([graphbase '.laplacian.commute-small']);
n = size(A,1);
deg = sum(A,1)';


%% Pick a set of random nodes and plot the precision of finding the closest non-neigbhor nodes
[ignore p] = sort(-deg);
l10s = log10seq(n);
degnodes = p(l10s); % get a sampling of the degree distribution.
nrandnodes = 50;
rand('state',0);
randnodes = randperm(n);
nodes = [degnodes(:)' randnodes(1:nrandnodes)];
nnodes = length(nodes);
ks = [5:5:100];
targetk = 25;
precisions_ap = zeros(nnodes, length(ks));
precisions_F = zeros(nnodes, length(ks));
precisions_D = precisions_F;
recall_D = precisions_D;
recall_F = precisions_F;
recall_ap = precisions_ap;
for ni=1:length(nodes)
    curn = nodes(ni);
    c = C(:,curn);
    c(c==0) = max(c);
    t0 = tic;
    [ap,d,reshist,x] = commute_topk(A,curn,'tol',1e-16); % approx commute
    dt_l = toc(t0);
    t0 = tic;
    %fv = 1./deg;
    %[fv,stats,xp] = F_topk(A,curn); % F score
    %x = x-min(x);
    %fv = -(deg.*x + (deg(curn)).*x);
    %fv = -x;
    fv = 2./deg+x(curn)-2*x;
    dt_p = toc(t0);
    d = 1./deg;
    fprintf('Runtime for node %5i (%2i/%2i): lanczos=%5.2f sec   lanczos_f=%5.2f sec\n', ...
        curn, ni, nnodes, dt_l, dt_p);
    
%     c(A(curn,:)>0) = max(c);
%     ap(A(curn,:)>0) = max(ap);
%     fv(A(curn,:)>0) = max(fv);
%     d(A(curn,:)>0) = max(d);

    c(curn) = max(c);
    ap(curn) = max(ap);
    fv(curn) = max(fv);
    d(curn) = max(d);

    
    [ignore pC] = sort(c);
    [ignore pL] = sort(ap);
    [ignore pF] = sort(fv);
    [ignore pD] = sort(d);
    
    for ki=1:length(ks)
        k = ks(ki);
        %precisions(ni,ki) = length(intersect(pL(1:k),pC(1:k)))/k;
        recall_ap(ni,ki) = length(intersect(pL(1:k),pC(1:targetk)))/min(k,targetk);
        recall_F(ni,ki) = length(intersect(pF(1:k),pC(1:targetk)))/min(k,targetk);
        recall_D(ni,ki) = length(intersect(pD(1:k),pC(1:targetk)))/min(k,targetk);
        precisions_ap(ni,ki) = length(intersect(pL(1:k),pC(1:k)))/min(k,k);
        precisions_F(ni,ki) = length(intersect(pF(1:k),pC(1:k)))/min(k,k);
        precisions_D(ni,ki) = length(intersect(pD(1:k),pC(1:k)))/min(k,k);
    end
    
    save(sprintf('%s-large-precision-results.mat',graphname),...
        'precisions_F','precisions_ap','precisions_D',...
        'recall_ap','recall_F','recall_D','nodes','targetk','ks','nrandnodes');
end
commute_topk_largegraph_plot