%% Evaluate the precision of our approximate topk commute time
% For a real-world graph, we see how badly the inaccurate diagonal
% estimation from cgLanczos affects the precision in the top 100 commute
% time neighbors.

addpath('../../matlab');
%% Real world graph
A = readSMAT('../../data/stanford-cs-sym.smat');
C = load_scalapack_matrix('../../data/stanford-cs-sym.smat.laplacian.ctimes.gz');
dexact = load('../../data/stanford-cs-sym.smat.laplacian.pinvdiags');
n = size(A,1);
deg = sum(A,1)';
graphname = 'cs-stan';

%% Setup figures
set(0, ...
    'defaultaxesfontsize',   16, ...
    'defaultaxeslinewidth',   1.5, ...
    'defaultlinelinewidth',   2, ...
    'defaultpatchlinewidth',  1.1);

%% Pick a set of random nodes and plot the precision
% The idea is that you run ONE of the various approximations above
% and show it's performance with this section.  
nnodes = 50;
rand('state',0);
randnodes = randperm(n);
nodes = randnodes(1:nnodes);
ks = [5:5:50];
precisions = zeros(nnodes, length(ks));
davg = 0;
for ni=1:nnodes
    curn = nodes(ni);
    [cv,dest,reshist] = commute_topk_raw(A,curn,'tol',1e-15);
    davg = davg + 1/nnodes*dest;
    [ignore pL] = sort(cv);
    [ignore pC] = sort(C(:,curn));
    for ki=1:length(ks)
        k = ks(ki);
        precisions(ni,ki) = length(intersect(pL(1:k),pC(1:k)))/k;
    end
    precisions(ni,:)
end
% Plot mean average precisions
%plot(ks,mean(precisions))
boxplot(precisions,'labels',ks)
ylim([-0.1,1.1])
xlabel('k'); ylabel(sprintf('Precision at k'));
%print(gcf,sprintf('%s-commute-vs-approx-commute-precision-%i-trials.eps', graphname, nnodes),'-depsc2');

%% Pick a set of random nodes and plot the precision
% The idea is that you run ONE of the various approximations above
% and show it's performance with this section.  
nnodes = 300;
rand('state',0);
randnodes = randperm(n);
nodes = randnodes(1:nnodes);
ks = [5:5:100];
targetk = 25;
precisions = zeros(nnodes, length(ks));
for ni=1:nnodes
    curn = nodes(ni);
    cv = commute_topk(A,curn,'tol',1e-10,'reorth',0);
    [ignore pL] = sort(cv);
    [ignore pC] = sort(C(:,curn));
    for ki=1:length(ks)
        k = ks(ki);
        precisions(ni,ki) = length(intersect(pL(1:k),pC(1:targetk)))/min(k,targetk);
    end
end
% Plot mean average precisions
%plot(ks,mean(precisions))
boxplot(precisions,'labels',ks)
ylim([-0.1,1.1])
xlabel('k'); ylabel(sprintf('Precision at k up to 25 correct'));
%print(gcf,sprintf('%s-commute-vs-approx-commute-precision-%i-trials.eps', graphname, nnodes),'-depsc2');


%% Pick a set of random nodes and plot the precision (without the neighbors)
% The idea is that you run ONE of the various approximations above
% and show it's performance with this section.  
nnodes = 300;
rand('state',0);
randnodes = randperm(n);
nodes = randnodes(1:nnodes);
ks = [5:5:100];
precisions = zeros(nnodes, length(ks));
for ni=1:nnodes
    curn = nodes(ni);
    [t,res,d] = commute_topk(A,curn,'tol',1e-10);
    c = C(:,curn);
    t(A(curn,:)>0) = max(t);
    c(A(curn,:)>0) = max(c);
    [ignore pL] = sort(t);
    [ignore pC] = sort(c);
    for ki=1:length(ks)
        k = ks(ki);
        %precisions(ni,ki) = length(intersect(pL(1:k),pC(1:k)))/k;
        precisions(ni,ki) = length(intersect(pL(1:k),pC(1:targetk)))/min(k,targetk);
    end
end
% Plot mean average precisions
%plot(ks,mean(precisions))
boxplot(precisions,'labels',ks)
ylim([-0.1,1.1])
xlabel('k'); ylabel(sprintf('Precision at k'));
print(gcf,sprintf('%s-commute-vs-approx-commute-precision-noneigh-%i-trials.eps', graphname, nnodes),'-depsc2');