function [V,d,A] = eigenmap(filename,type,nvecs)
% EIGENMAP Compute a Laplacian eigenmap
%
% [V,d] = eigenmap(A,type,nvecs)
% if A is a sparse matrix, then this call will use that matrix
% otherwise, it reads in a sparse matrix from filename (readSMAT), 
% symmetrices it via max(A,A'), then finds the nvecs smallest eigenvalues
% and vectors for the Laplacian matrix.  If type == 'normalized', 
% then it will use the normalized Laplacian instead.
%
% V are the eigenvectors and d are the eigenvalues

if ~exist('type','var') || isempty(type), type = 'standard'; end
if ~exist('nvecs','var') || isempty(nvecs), nvecs = 10; end

Ad = readSMAT(filename);
A = max(Ad,Ad');

if strcmp(type,'normalized')
    L = nlaplacian(A);
else
    L = laplacian(A);
end

[V,D] = eigs(L,nvecs+1,'SA');
% remove the singular vector
V = V(:,2:end);
d = diag(D);
d = d(2:end);


