%% Output basic information about the graphs we use

%% Basic setup
% First add information to the paths
addpath('../../matlab'); % add the directory with the katz_push algorithm
addpath('../../gaimc');
addpath('../../tools');

%%
% Set the graphs
data_dir = '../../data';
graphs = {...
    'stanford-cs-sym.smat'
    'wiki-Vote-wcc.smat'
    'hollywood-2009-cc.smat'
    'flickr2.smat'
    'arxiv_lcc.smat'
    'ca-AstroPh-cc.smat'
    'ca-CondMat-cc.smat'
    'ca-GrQc-cc.smat'
    'ca-HepPh-cc.smat'
    'ca-HepTh-cc.smat'
    'dblp_lcc.smat'
    'email-Enron-cc.smat'
    'email-EuAll-wcc.smat'
    'soc-Epinions1-wcc.smat'
    'soc-Slashdot0811-wcc.smat'
    'tapir.smat'
    'Stanford3-cc.smat'
    };

%% Graph properties
clear result;
results = [];
for gi = 1:length(graphs)
    graph = graphs{gi};
    graphfile = fullfile(data_dir, [graph]);
    A = readSMAT(graphfile);
    A = prepare_graph(A);
    n = size(A,1);
    m = nnz(A);
    
    result.name = graph;
    result.nverts = n;
    result.nedges = m/2;
    result.avgdeg = m/n;
    
    degs = sum(A,2);
    result.maxdeg = max(degs);
    
    sigma = max(svds(A));
    result.sigma = sigma;
    
    cn = corenums(A);
    result.twocore = sum(cn>=2);
    
    svdt=tic;
    [U,S,V] = svds(A,200);
    result.svdtime = toc(svdt);
    
    if isempty(results), results = result; 
    else  results(end+1) = result;
    end
    result
    curg = gi;
    
    save 'graphdata.mat' results graphs curg
end

%%
load graphdata
[ignore p] = sort([results.nedges]);
for ri=1:length(results)
    result = results(p(ri));
    name = adjust_graphname(result.name);
    fprintf('%20s & %7i & %8i & %6.2f & %5i & %9.4f & %.1f \\\\ \n',...
        name, result.nverts, result.nedges, result.avgdeg, full(result.maxdeg), ...
        result.sigma, result.svdtime);
end
