
%% Setup figures
close all;
set(0,'DefaultAxesLineWidth',0.75);
set(0,'DefaultLineLineWidth',1.1);
set(0,'DefaultAxesFontSize',8);

%% Generate a results filename
vert = pairs_conv(2);
rfilename = sprintf('accuracy-results-%s-katz-%s-%i',graphname,katz_alpha_name,vert);

%% Load the data we just saved
load(rfilename);

%% Plot the convergence in the ordering

%setup_plot;

clf; hold on;
allktau = [results(1:length(results)).ktauk];
alloverlap = [results(1:length(results)).overlapk];

xs = [results(1:length(results)).nedges]/nnz(A);
legtexts = {};
hs=[];
styles = {'.-','.--','.-.','.:'};
for ki=1:length(ks);
    hs(ki) = semilogx(xs,allktau(ki,:),styles{ki},'MarkerSize',14,...
        'Color',0.5*[1,1,1],'linewidth',1.1);
    % againth, the space is important'
    %text(xs(1),alloverlap(ki,1),sprintf('k=%i ',ks(ki)),'HorizontalAlignment','right');
    legtexts{ki} = sprintf('k=%i ',ks(ki));
end
% double-print k=25
hs(end+1) = semilogx([cgresults.nedges]/nnz(A),[cgresults.ktauk],'ko-');
legtexts{end+1} = 'cg k=25 ';
ki=2;
hs(end+1) = semilogx(xs,allktau(ki,:),styles{ki},'MarkerSize',14,'Color','r');
legtexts{end+1} = sprintf('k=%i ',ks(ki));
legtexts(ki)=[];
hs(ki)=[];
title(sprintf('%s, Katz, %s alpha',graphname,katz_alpha_name));

ylim([-0.1,1.1]);
xlim([1e-3,1e3]);
set(gca,'XTick',[1e-2,1e-1,1,10,100]);
lh = legend(hs,legtexts{:},'Location','SE');
legend boxoff;
set(gca,'XScale','log');
xlabel('Equivalent matrix-vector products');
ylabel('Kendtall-\tau ordering vs. exact');

%%
box on;
greygrid;
transfigure;
set_figure_size('small');
print(gcf,sprintf('katz-%s-%s-topk-ktau-%i.eps',katz_alpha_name,graphname,vert),'-depsc2');

%% Plot the convergence in the sets
clf; hold on;
alloverlap = [results(1:length(results)).overlapk];
xs = [results(1:length(results)).nedges]/nnz(A);
legtexts = {};
hs=[];
styles = {'.-','.--','.-.','.:'};
for ki=1:length(ks);
    hs(ki) = semilogx(xs,alloverlap(ki,:),styles{ki},'MarkerSize',14,...
        'Color',0.5*[1,1,1],'linewidth',1.1);
    % againth, the space is important'
    %text(xs(1),alloverlap(ki,1),sprintf('k=%i ',ks(ki)),'HorizontalAlignment','right');
    legtexts{ki} = sprintf('k=%i ',ks(ki));
end
% double-print k=25
hs(end+1) = semilogx([cgresults.nedges]/nnz(A),[cgresults.overlapk],'ko-');
legtexts{end+1} = 'cg k=25 ';
ki=2;
hs(end+1) = semilogx(xs,alloverlap(ki,:),styles{ki},'MarkerSize',14,'Color','r');
legtexts{end+1} = sprintf('k=%i ',ks(ki));
legtexts(ki)=[];
hs(ki)=[];
set(gca,'XScale','log');
ylim([-0.1,1.1]);
xlim([1e-3,1e3]);
set(gca,'XTick',[1e-2,1e-1,1,10,100]);
lh = legend(hs,legtexts{:},'Location','SE');
legend boxoff;
title(sprintf('%s, Katz, %s alpha',graphname,katz_alpha_name));

xlabel('Equivalent matrix-vector products');
ylabel('Precision@k for exact top-k sets');

%%
box on;
greygrid;
transfigure;
set_figure_size('small');
print(gcf,sprintf('katz-%s-%s-topk-overlap-%i.eps',katz_alpha_name,graphname,vert),'-depsc2');