%% Compare topk_katz with balanced pushes vs. normal pushes
% The two metrics are: number of vertices explored, and number of pushes.

%% Setup

addpath('../../matlab');

%% Parameters

data_dir = '../../data';
graphs = {...
    %'Stanford3-cc.smat'
    'arxiv_lcc.smat'
    'ca-AstroPh-cc.smat'
    'ca-CondMat-cc.smat'
    'ca-GrQc-cc.smat'
    'ca-HepPh-cc.smat'
    'ca-HepTh-cc.smat'
    'dblp_lcc.smat'
    'email-Enron-cc.smat'
    'email-EuAll-wcc.smat'
    'flickr2.smat'
    'hollywood-2009-cc.smat'
    'soc-Epinions1-wcc.smat'
    'soc-Slashdot0811-wcc.smat'
    'stanford-cs-sym.smat'
    'tapir.smat'
    'wiki-Vote-wcc.smat'
    };
%graphs = {'email-Enron-cc.smat'};
%%
nrandverts = 50;
degvertseq = [1 2 3 4 5];
degvertssets = [degvertseq ...
    10*degvertseq ...
    100*degvertseq ... 
    1000*degvertseq ...
    10000*degvertseq ...
    100000*degvertseq ...
    1000000*degvertseq];

%%
for gi=1:length(graphs)
    gname = graphs{gi};
    fprintf('\n');
    fprintf('\n');
    fprintf('%s\n', gname);
    fprintf('\n');
    A = readSMAT(fullfile(data_dir,gname));
    n = size(A,1);
    degs = full(sum(A,2));
    fprintf('vertices: %i\n', size(A,1));
    fprintf('nonzeros: %i\n', nnz(A));
    fprintf('max degree: %i\n',max(degs));
    % find a reasonable alpha for Katz
    sigma = max(svds(A));
    alpha = 1/(sigma + 1);
    fprintf('max sigma: %g\n', sigma);
    fprintf('Katz alpha: %g\n', alpha);
    fprintf('\n');
    % find an interesting set of vertices
    % dverts sample the degree distribution
    % rverts are just random samples
    [ignore p] = sort(degs,'descend');
    dverts = p(degvertssets(degvertssets < n));
    randverts = randperm(n);
    rverts = randverts(1:nrandverts);
    qverts = [dverts(:)' rverts(:)'];
    %%
    for vi=qverts
        % vi is the query vertex
        b = zeros(n,1);
        b(vi) = 1;
        [xtrue,flag,relres,iter] = pcg(speye(n) - alpha*A,b,1e-12,9999);
        %%
        t0 = tic;
        [xp,r,histp,nvertsp] = katz_push_mex(A,alpha,vi,1e-4,35*n);
        dtp = toc(t0);
        t0 = tic;
        [xb,r,histb,nvertsb] = katz_push_bal_mex(A,alpha,vi,1e-4,35*n);
        dtb = toc(t0);
        steps_ratio = length(histb)/length(histp);
        verts_ratio = nvertsb/nvertsp;
        edges_ratio = histb(1,end)/histp(1,end);
        %steps_ratio = dtb/dtp;
        fprintf('vertex=%5i deg=%5i pcgiter=%4i errpush=%8.1e errbal=%8.1e edgerat=%10g (%4.1f) vertsdiff=%10g (%4.1f)\n',...
            vi, degs(vi), iter, norm(xp-xtrue), norm(xb-xtrue), edges_ratio, histp(1,end)/nnz(A), verts_ratio, 100*nvertsp/n);
    end
end