%% Compare matrix-vector performance to two-in-one matvec performance
% Is doing A*[x y] really take twice as long as A*x?  Let's find out!

%% Compile
mex -largeArrayDims matvec_mex.c
mex -largeArrayDims dualmatvec_mex.c
mex -largeArrayDims twomatvec_mex.c

%% Setup
addpath('../../matlab');
A = readSMAT('../../data/hollywood-2009-cc.smat');
x = randn(size(A,1),1);
ntrials = 20;

%% One-at-a-time
ytrue = A'*x;
[y,dt1] = matvec_mex(A,x,ntrials);
norm(y-ytrue)

%% Two-at-a-time
% Assume X is stored by row.
X = [x,x];
Xt = reshape(X,2,size(A,1));
Ztrue = A'*Xt';
[Z,dt2] = dualmatvec_mex(A,X,ntrials);
Zt = reshape(Z,2,size(A,1));
norm(Zt'-Ztrue,'fro')

%% Two-at-a-time
% Assume X is stored by column
X = [x,x];
Ytrue = A'*X;
[Y,dt2c] = twomatvec_mex(A,X,ntrials);
norm(Y-Ytrue,'fro')

%% Summarize
DT = [dt1; dt2; dt2c]';
mean(DT)
std(DT)
%% 
% On mandelbrot on 3/24/2011, I got
% ans =
% 
%     0.2307    0.3591    0.4404
% 
% 
% ans =
% 
%     0.0005    0.0011    0.0010
