%% Run the comparison for CG vs. MMQ-Commute
% 

addpath('../../matlab');
addpath('../../tools');

%% load a graph
if ~exist('graphname','var') || ~exist('A','var')
    error('need to load a graph first another function');
end

%% normalize the graph
A = A|A';
A = A - diag(diag(A));
deg = sum(A,1)'; 
n = size(A,1);

%% Pick vertices
[ignore p] = sort(deg,'descend');
vdeg = p(log10seq(n,[2]));
nrandpairs = 20;
pairs = zeros(length(vdeg)*(length(vdeg)-1)/2+nrandpairs,2);
pi = 1;
for i=1:length(vdeg)
    for j=i+1:length(vdeg)
        pairs(pi,:) = [vdeg(i) vdeg(j)];
        pi = pi + 1;
    end
end
rand('state',0);
randverts = randperm(n);
assert(n/2 >= nrandpairs); % otherwise, not enough random pairs!
for i=1:nrandpairs
    pairs(pi,:) = [randverts(2*i-1) randverts(2*i)];
    pi = pi + 1;
end
assert(pairs(end,1) > 0); % make sure we got them all

npair = size(pairs,1);
fprintf('vertex pairs: %i\n', npair);

%% Compute all the pairs
ratios = zeros(npair,1);
cgmatvecs = zeros(npair,1);
mmqmatvecs = zeros(npair,1);
cg_results = zeros(npair,1);
mmq_results = zeros(npair,1);
mmq_time = zeros(npair,1);
cg_time = zeros(npair,1);
curt0 = tic;
for pi=1:size(pairs,1)
    %%
    vi = pairs(pi,1);
    vj = pairs(pi,2);

    %% Get bounds from Golub-Meurant algorithm
    t0 = tic;
    [result bounds time nmatvec] = commute_pairwise(A,vi,vj,1e-4,1000,1e-4);
    mmq_dt = toc(t0);
    mmq_results(pi) = result;

    %% Compare against CG
    t0=tic;
    [ccg cg_hist] = commute_pairwise_cg(A,vi,vj,1e-4,1000);
    cg_dt=toc(t0);
    
    cg_results(pi) = ccg;
    
    cgmatvecs(pi) = length(cg_hist);
    mmqmatvecs(pi) = nmatvec;
    mmq_time(pi) = mmq_dt;
    cg_time(pi) = cg_dt;
    
    % ratio is (ref - exp)/ref)
    
    perfratio = (cgmatvecs(pi)-mmqmatvecs(pi))/(cgmatvecs(pi));
    ratios(pi) = perfratio;
    
    %% Status
    fprintf('.');
    if mod(pi,50)==0
        fprintf(' ');
        dt = toc(curt0);
        tr = (dt/pi)*npair - dt;
        fprintf('%3i/%3i = %4.1f%% dt:%.1f tr:%.1f', pi, npair,...
            100*pi/size(pairs,1), dt, tr);
        fprintf('\n');
    end
end
fprintf('\n');
