%% Compare pcg to katz-pairwise
% In this experimental sweep over many graphs, we compare CG to the Katz
% pairwise MMQ algorithm.  The goal is to see how many matvecs each takes
% as we vary the graph.


%% Setup
addpath('../../matlab');

%% Parameters

data_dir = '../../data';
graphs = {...
    'stanford-cs-sym.smat'
    'wiki-Vote-wcc.smat'
    'hollywood-2009-cc.smat'
    'flickr2.smat'
    'arxiv_lcc.smat'
    'ca-AstroPh-cc.smat'
    'ca-CondMat-cc.smat'
    'ca-GrQc-cc.smat'
    'ca-HepPh-cc.smat'
    'ca-HepTh-cc.smat'
    'dblp_lcc.smat'
    'email-Enron-cc.smat'
    'email-EuAll-wcc.smat'
    'soc-Epinions1-wcc.smat'
    'soc-Slashdot0811-wcc.smat'
    'tapir.smat'
    'Stanford3-cc.smat'
    };


nrandpairs = 100;

%%
try
    load 'katz_pairwise_mv_results.mat';
catch m
    curg = 0;
end

%%
results = [];
for gi=(curg+1):length(graphs)
    gname = graphs{gi};
    graphname = gname;
    fprintf('\n');
    fprintf('\n');
    fprintf('%s\n', gname);
    fprintf('\n');
    A = readSMAT(fullfile(data_dir,gname));
    n = size(A,1);
    degs = full(sum(A,2));
    fprintf('vertices: %i\n', size(A,1));
    fprintf('nonzeros: %i\n', nnz(A));
    fprintf('max degree: %i\n',max(degs));
    % find a reasonable alpha for Katz
    sigma = max(svds(A));
    alpha = 1/(sigma + 1);
    fprintf('max sigma: %g\n', sigma);
    fprintf('Katz alpha: %g\n', alpha);
    fprintf('\n');
    
    katz_alpha = alpha;
    katz_alpha_name = 'hard';
    
    fprintf('Computing katz scores for vertex pairs.\n');
    
    pairs_t0 = tic;
    katz_pairwise_matvecs;
    dt = toc(pairs_t0);
    fprintf('Done. (%.1f sec)\n',dt);
    
    % we now have pairs, cgmatvecs, mmqmatvecs, and ratios
    results(gi).name = gname;
    results(gi).nverts = size(A,1);
    results(gi).nedges = nnz(A);
    results(gi).maxdeg = max(degs);
    results(gi).sigma = sigma;
    results(gi).alpha = alpha;
    
    results(gi).pairs = pairs;
    
    results(gi).cgmatvecs = cgmatvecs;
    results(gi).mmqmatvecs = mmqmatvecs;
    results(gi).ratios = ratios;
    results(gi).mmq_results = mmq_results;
    results(gi).cg_results = cg_results;
    results(gi).mmq_time = mmq_time;
    results(gi).cg_time = cg_time;
    
    curg = gi;
    save 'katz_pairwise_mv_results.mat' results curg;
end

